/******************************************************************************
 * Graph.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef GRAPH_H_
#define GRAPH_H_

#include <vector>
#include <utility>

using namespace std;


typedef pair<int,int> Edge;

/**
 * Representation of a graph.
 */
class Graph {

private:
	vector<int> conNodes;
    vector<Edge> conEdges;
public:
	Graph();
	virtual ~Graph();

	void addNode(int node){
		 conNodes.push_back(node);
	}
	void addEdge(int node1, int node2){
		conEdges.push_back(pair<int,int>(node1,node2));
	}

	bool areAdjacent(int node1, int node2){
		for(int i=0; i < m(); ++i){
			if((conEdges[i].first == node1 && conEdges[i].second == node2) ||
			   (conEdges[i].first == node2 && conEdges[i].second == node1)){
				return true;
			}
		}
		return false;
	}

	int getEdge(int node1, int node2){
		for(int i=0; i < m(); ++i){
			if((conEdges[i].first == node1 && conEdges[i].second == node2) ||
			   (conEdges[i].first == node2 && conEdges[i].second == node1)){
				return i;
			}
		}
		return -1;
	}

	int n() {return conNodes.size();}
	int m() {return conEdges.size();}

	vector<int> & nodes(){ return conNodes;}
	vector<Edge > & edges(){ return conEdges;}

};

#endif /* GRAPH_H_ */
