/******************************************************************************
 * ProblemSolver.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
#include "ProblemSolver.h"

ProblemSolver::ProblemSolver() {}

ProblemSolver::~ProblemSolver() {}



void ProblemSolver::execute(Graph & graph,  Result & solution, Config * config){
	SolverResult result;
	int turn =0;
	do{
		ProblemModel * problemModel = createModel(graph,solution.preClock,turn, config);

		solution.model = problemModel;


		SATSolver solver;

		solution.solverClock.start();
		result = solver.execute(problemModel->getCNFModel());
		solution.solverClock.stop();

		++turn;

		if(analyze(result,problemModel,solution) || result == TIME_OUT){
			solution.status = result;
			return;
		}

		delete problemModel;
	}while(nextTurn(graph,turn,result));
	solution.status = NOT_SOLVED;
	solution.model =0;
}


ClosedInterval ProblemModel::getInterval(Range & variables){
	ClosedInterval interval;
	interval.first = interval.second =0;
	int state =0;
	int pos =0;
	for(int i=variables.begin; i < variables.end; ++i,++pos){
			if(getCNFModel().getValue(i)){
				if(state == 0){
					state = 1;
					interval.first = pos;
				}else if(state == 2){
					cout << "Pos: " << i-variables.begin << endl;
					throw "There is more than one interval.";
				}

			}else{
				if(state == 1){
					state =2;
					interval.second = pos;
				}
			}
	}
	if(state == 1){
		throw "Interval has not been closed.";
	}
	interval.second = interval.second-1; // right bound of interval is inclusive.
	return interval;
}
