/******************************************************************************
 * SATSolver.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include <sys/wait.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <fstream>
#include "SATSolver.h"

#define MAX PIPE_BUF*8

string SATSolver::tempFile="/home/benjamin/temp/test";
string SATSolver::addArgs="-cpu-lim=300";
string SATSolver::command="~/programs/minisat/core/minisat";

enum PIPE_FILE_DESCRIPTERS
{
  READ_FD  = 0,
  WRITE_FD = 1
};

enum CONSTANTS
{
  BUFFER_SIZE = 100
};



SATSolver::SATSolver() {
	// TODO Auto-generated constructor stub

}

SATSolver::~SATSolver() {
	// TODO Auto-generated destructor stub
}


SolverResult SATSolver::execute(CNFModel & model){

	stringstream output;

	string outFile = tempFile+".model.cnf";
	string inFile  = tempFile+".result.sat";

	fstream file;
	file.open(outFile.c_str(), ios::out |ios::trunc);
	 if (file.is_open())
	  {
		model.write(file);
		file.close();
	  } else {
		  cout << "Unable to open file " <<outFile;
		  throw "could not open file";
	  }

	 string command = SATSolver::command+" " +addArgs +" "+ outFile +" "+ inFile;


	 FILE * result = popen(command.c_str(),"r");


	 char buffer [256];


	 stringstream info;

	 if (result == NULL) throw "could not open pipe";
	 else
	 {
	   while ( ! feof (result) )
	   {
	     if ( fgets (buffer , 256 , result) == NULL ) break;
	     info << buffer;
	   }
	   fclose (result);
	 }

	 string execInfo = info.str();

	 if(execInfo.find("UNSATISFIABLE",0)!=string::npos){
		 return UNSAT;
	 }else if(execInfo.find("SATISFIABLE",0)!=string::npos){
		fstream resultFile;
		resultFile.open(inFile.c_str(), ios::in);

		model.interpret(resultFile);
		return SAT;

	 }else if(execInfo.find("INDETERMINATE",0)){
		return TIME_OUT;
	 }
	 throw "some error occurred.";



}





