/******************************************************************************
 * SATSolver.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef SATSOLVER_H_
#define SATSOLVER_H_

#include "Model.h"

enum SolverResult {SAT,UNSAT, TIME_OUT, NOT_SOLVED};


/**
 * Represents the interface of the SAT solver used for solving
 * a CNF model.
 */
class SATSolver {


public:

	/**
	 * path of the temporary file that is created when the sat solver is
	 * executed.
	 */
	static string tempFile;
	/**
	 * Additional arguments for the SAT solver separated by spaces.
	 */
	static string addArgs;
	/**
	 * execution command of the SAT solver.
	 */
	static string command;

	SATSolver();
	virtual ~SATSolver();

	/**
	 * Method executes SAT solver on the given model.
	 */
	SolverResult execute(CNFModel & model);
};

#endif /* SATSOLVER_H_ */
