/******************************************************************************
 * Bandwidth.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
#include <iostream>

#include "Bandwidth.h"
#include "../common/Graph.h"
#include "../common/Model.h"
#include "../common/Box.h"
#include "../common/SATSolver.h"

using namespace std;



void createDistanceConstraints(CNFModel & model, Graph & graph, Range * nodes, int length, int u, int v, int p){
	for(int i=0; i < length; ++i){
		for(int j=i+p+1; j < length; ++j){
			Clause * cnf = model.next(2);
			cnf->add(nodes[u].getVar(i),false);
			cnf->add(nodes[v].getVar(j),false);
		}

	}


}


BWModel* createBWModel(int bandwidth,Graph & graph, Clock& clock){
		clock.start();

		// length of grid.
		int length = graph.n();

		// upper bound for variables and clauses.
		int varNumber = graph.n()*graph.n();
		int cnfNumber =   graph.n()
						+ graph.n()*(graph.n()-1)/2*length
						+ 2*graph.n()*length*length;
				;

		BWModel * bwModel = new BWModel(bandwidth,varNumber,cnfNumber);

		CNFModel & model = bwModel->getCNFModel();

	    Range * nodes = new Range[graph.n()];
	    bwModel->nodes = nodes;


	    for(int i=0; i < graph.n(); ++i){
	    	nodes[i] = model.reserveVariables(length);
	    }

	    // CONSTRAINT (5): Each node occupies at least one grid point.
		for(int i=0; i < graph.n(); ++i){
			Clause* cnf = model.next(length);

			for(int j=0; j < length; ++j){
				cnf->add(nodes[i].getVar(j),true);
			}
		}

	    // CONSTRAINT (22): Nodes do not overlap.
		for(int i=0; i < graph.n(); ++i){
			for(int j=i+1; j < graph.n(); ++j){
				for(int k=0; k < length; ++k){
					Clause * cnf = model.next(2);
					cnf->add(nodes[i].getVar(k),false);
					cnf->add(nodes[j].getVar(k),false);
				}
			}
		}

		// CONSTRAINT (23): Adjacent nodes have distance less than or equal to p.
		for(int i=0; i < graph.m(); ++i){
			pair<int,int> edge = graph.edges()[i];
			createDistanceConstraints(model,graph,nodes,length,edge.first,edge.second,bandwidth);
			createDistanceConstraints(model,graph,nodes,length,edge.second,edge.first,bandwidth);
		}
		clock.stop();
		return bwModel;
}

void BWModel::checkForFeasibility(Graph & graph){

	cout << "START FEASIBILITY TESTS FOR BANDWIDTH MODEL" << endl;
		CNFModel & model = getCNFModel();
		int length = graph.n();
		int * order = new int[length];
				int * inverse = new int[length];
				for(int i=0; i< length; ++i){
					order[i]= -1;
					inverse[i]=-1;
				}

				for(int i=0; i < graph.n(); ++i){

					for(int j=0; j < length; ++j){
						if(model.getValue(nodes[i].getVar(j))){
							order[j]=i;
							inverse[i]=j;
							cout << "1 ";
						}else{
							cout << "0 ";
						}
					}
					cout << endl;
				}

				cout << "order: " << endl;
				for(int i=0; i < length; ++i){
					cout << order[i] << " ";
				}
				cout << endl;


				for(int i=0; i< length; ++i){
					if(order[i]==-1 || inverse[i]==-1){
						cout << "node is missing" << endl;
						throw "node is missing";
					}
				}

				for(int i=0; i < graph.m(); ++i){
					pair<int, int> edge = graph.edges()[i];
					bool found = false;
					for(int j=inverse[edge.first]+1; j <= inverse[edge.first]+bandwidth; j++){
						if(order[j]==edge.second){
							found = true;
						}
					}
					for(int j=inverse[edge.second]+1; j <= inverse[edge.second]+bandwidth; j++){
								if(order[j]==edge.first){
									found = true;
								}
					}
					if(!found){
						cout << "not within a range of p nodes" << endl;
						throw "not within a range of p nodes";
					}
				}
				cout << "test has been successfully processed" << endl;
}





