/******************************************************************************
 * Bandwidth.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef BANDWIDTH_NEW_H_
#define BANDWIDTH_NEW_H_

#include "../common/Graph.h"
#include "../common/Clock.h"
#include "../common/ProblemSolver.h"


#include "../common/SATSolver.h"

class BWModel;

/**
 * Creates a CNF model that is satisfied if and only if the graph has the given bandwidth or less.
 */
BWModel * createBWModel(int bandwidth, Graph & graph, Clock& clock);



/**
 * Stores an CNF model and its corresponding interpretation.
 * The array nodes contains for each node the range of variables.
 */
class BWModel : public ProblemModel{
public:
	Range * nodes;
	int     bandwidth;

	BWModel(int bandwidth,int varCount, int cnfCount) : ProblemModel(varCount,cnfCount), bandwidth(bandwidth){

	}

	virtual ~BWModel(){
		delete [] nodes;
	}

	virtual int getValue(){
		return bandwidth;
	}

	virtual void checkForFeasibility(Graph & graph);
};


class BandwidthSolver : public ProblemSolver{


	virtual ProblemModel * createModel(Graph & graph,Clock & clock, int turn, Config * config){

		BWModel * model = createBWModel(turn+1,graph,clock);
		model->bandwidth = turn+1;
		return model;
	}

	virtual bool  analyze(SolverResult & result, ProblemModel * model, Result & solution){
		return result == SAT;
	}

	virtual bool  nextTurn(Graph & graph, int currentTurn, SolverResult & result){
		return currentTurn <= graph.n();

	}
};

#endif /* BANDWIDTH_H_ */
