/******************************************************************************
 * Boxicitiy.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef BOXICITY_H_
#define BOXICITY_H_

#include "./../common/Graph.h"
#include "./../common/Clock.h"
#include "./../common/SATSolver.h"
#include "./../common/ProblemSolver.h"
#include "./../common/Box.h"

class BCModel;


/**
 * Creates a CNF model that is satisfied if and only if the boxicity representation
 * of the given graph can be drawn on a grid of size width x height.
 */
BCModel * createBCModel(int width, int height, Graph & graph, Clock& clock);


/**
 * Contains the CNFModel created by createBCModel and the interpretation of the variables, i.e.,
 * the relation between nodes and variables.
 */
class BCModel : public ProblemModel{
	void print(Graph & graph);
public:

	Box   * nodes;
	int     width;
	int 	height;

	BCModel(int varCount, int cnfCount) : ProblemModel(varCount,cnfCount) {

	}

	virtual ~BCModel(){
		delete [] nodes;
	}

	virtual void checkForFeasibility(Graph & graph);

	virtual int getValue(){
		return width;
	}
};



class BCSolver : public ProblemSolver{


	virtual ProblemModel * createModel(Graph & graph,Clock & clock, int turn, Config * config){
		return createBCModel(turn+1,turn+1,graph,clock);
	}

	virtual bool  analyze(SolverResult & result, ProblemModel * model, Result & solution){
		return result == SAT;
	}

	virtual bool  nextTurn(Graph & graph, int currentTurn, SolverResult & result){
		return currentTurn <= graph.n();

	}
};

#endif /* BOXICITYSOLVER_H_ */
